interface IClassGeneratorContextItem {
    name: string;
    usedBy: Set<string>;
}
interface IClassGeneratorOptions {
    reserveClassName?: (string | RegExp)[];
    customGenerate?: (original: string, opts: IClassGeneratorOptions, context: Record<string, any>) => string | undefined;
    log?: boolean;
    exclude?: (string | RegExp)[];
    include?: (string | RegExp)[];
    ignoreClass?: (string | RegExp)[];
    classPrefix?: string;
}
interface IClassGenerator {
    newClassMap: Record<string, IClassGeneratorContextItem>;
    newClassSize: number;
    context: Record<string, any>;
}

declare class ClassGenerator implements IClassGenerator {
    newClassMap: Record<string, IClassGeneratorContextItem>;
    newClassSize: number;
    context: Record<string, any>;
    opts: IClassGeneratorOptions;
    classPrefix: string;
    constructor(opts?: IClassGeneratorOptions);
    defaultClassGenerate(): string;
    ignoreClassName(className: string): boolean;
    includeFilePath(filePath: string): boolean;
    excludeFilePath(filePath: string): boolean;
    isFileIncluded(filePath: string): boolean;
    transformCssClass(className: string): string;
    generateClassName(original: string): IClassGeneratorContextItem;
}

declare function escapeStringRegexp(str: string): string;
declare function makeRegex(str: string, options?: {
    exact: boolean;
}): RegExp;

declare const validateFilterRE: RegExp;
declare function isValidSelector(selector?: string): selector is string;
declare const splitCode: (code: string, options?: {
    splitQuote?: boolean;
}) => string[];

declare const preserveClassNames: string[];
declare const defaultMangleClassFilter: (className: string) => boolean;
declare function groupBy<T>(arr: T[], cb: (arg: T) => string): Record<string, T[]>;
declare const acceptChars: string[];
declare function stripEscapeSequence(words: string): string;
declare function isRegexp(value: unknown): boolean;
declare function isMap(value: unknown): boolean;
declare function regExpTest(arr: (string | RegExp)[] | undefined, str: string): boolean;

export { ClassGenerator, type IClassGenerator, type IClassGeneratorContextItem, type IClassGeneratorOptions, acceptChars, defaultMangleClassFilter, escapeStringRegexp, groupBy, isMap, isRegexp, isValidSelector, makeRegex, preserveClassNames, regExpTest, splitCode, stripEscapeSequence, validateFilterRE };
